@@ -1,10 +1,12 @@
 from __future__ import annotations
 
 import json
+from unittest import mock
 from unittest.mock import MagicMock, Mock
 
 from kombu.asynchronous.aws.ext import AWSRequest, boto3
-from kombu.asynchronous.aws.sqs.connection import AsyncSQSConnection
+from kombu.asynchronous.aws.sqs.connection import (AsyncSQSConnection,
+                                                   _query_object_encode)
 from kombu.asynchronous.aws.sqs.message import AsyncMessage
 from kombu.asynchronous.aws.sqs.queue import AsyncQueue
 from kombu.utils.uuid import uuid
@@ -23,14 +25,14 @@ class test_AsyncSQSConnection(AWSCase):
             aws_secret_access_key='AAAA',
             region_name='us-west-2',
         )
-        sqs_client = session.client('sqs')
-        self.x = AsyncSQSConnection(sqs_client, 'ak', 'sk', http_client=Mock())
+        self.sqs_client = session.client('sqs')
+        self.x = AsyncSQSConnection(self.sqs_client, 'ak', 'sk', http_client=Mock())
         self.x.get_object = Mock(name='X.get_object')
         self.x.get_status = Mock(name='X.get_status')
         self.x.get_list = Mock(name='X.get_list')
         self.callback = PromiseMock(name='callback')
 
-        sqs_client.get_queue_url = MagicMock(return_value={
+        self.sqs_client.get_queue_url = MagicMock(return_value={
             'QueueUrl': 'http://aws.com'
         })
 
@@ -71,14 +73,60 @@ class test_AsyncSQSConnection(AWSCase):
         assert req1.params == req2.params
         assert dict(req1.headers) == dict(req2.headers)
 
+    def test_fetch_attributes_on_construction(self):
+        """Verify default fetch_message_attributes can be set at construction."""
+        x = AsyncSQSConnection(
+            self.sqs_client, 'ak', 'sk', http_client=Mock(),
+            fetch_message_attributes=["AttributeOne"],
+        )
+        assert x.fetch_message_attributes == ["AttributeOne"]
+
+        # Default value for backwards compatibility
+        assert self.x.fetch_message_attributes == ["ApproximateReceiveCount"]
+
+    def test_create_query_request_get(self):
+        # Query Protocol GET call per
+        # https://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-making-api-requests-xml.html
+        operation_name = 'CreateQueue'
+        params = {
+            'DefaultVisibilityTimeout': 40,
+            'QueueName': 'celery-test',
+            'Version': '2012-11-05',
+        }
+        verb = 'GET'
+        req = self.x._create_query_request(operation_name, params, SQS_URL, verb)
+        self.assert_requests_equal(req, AWSRequest(
+            url=SQS_URL,
+            method=verb,
+            data=None,
+            params={
+                'Action': operation_name,
+                **params
+            },
+            headers={},
+        ))
+
+        prepared = req.prepare()  # without signing for test
+
+        assert prepared.method == 'GET'
+        assert prepared.url == (
+            'https://sqs.us-west-2.amazonaws.com/?'
+            'DefaultVisibilityTimeout=40'
+            '&QueueName=celery-test'
+            '&Version=2012-11-05'
+            '&Action=CreateQueue'
+        )
+        assert prepared.headers == {}
+        assert prepared.body is None
+
     def test_create_query_request(self):
-        operation_name = 'ReceiveMessage',
+        operation_name = 'ReceiveMessage'
         params = {
             'MaxNumberOfMessages': 10,
             'AttributeName.1': 'ApproximateReceiveCount',
             'WaitTimeSeconds': 20
         }
-        queue_url = f'{SQS_URL}/123456789012/celery-test'
+        queue_url = f'{SQS_URL}123456789012/celery-test'
         verb = 'POST'
         req = self.x._create_query_request(operation_name, params, queue_url,
                                            verb)
@@ -86,21 +134,38 @@ class test_AsyncSQSConnection(AWSCase):
             url=queue_url,
             method=verb,
             data={
-                'Action': (operation_name),
+                'Action': operation_name,
                 **params
             },
-            headers={},
+            headers={
+                'Content-Type': 'application/x-www-form-urlencoded; charset=utf-8',
+            },
         ))
 
+        prepared = req.prepare()  # without signing for test
+
+        assert prepared.method == 'POST'
+        assert prepared.url == queue_url
+        assert prepared.headers == {
+            'Content-Type': 'application/x-www-form-urlencoded; charset=utf-8',
+            'Content-Length': mock.ANY,
+        }
+        assert prepared.body == (
+            'MaxNumberOfMessages=10'
+            '&AttributeName.1=ApproximateReceiveCount'
+            '&WaitTimeSeconds=20'
+            '&Action=ReceiveMessage'
+        )
+
     def test_create_json_request(self):
         operation_name = 'ReceiveMessage'
         method = 'POST'
         params = {
             'MaxNumberOfMessages': 10,
-            'AttributeName.1': 'ApproximateReceiveCount',
+            'AttributeNames': ['ApproximateReceiveCount'],
             'WaitTimeSeconds': 20
         }
-        queue_url = f'{SQS_URL}/123456789012/celery-test'
+        queue_url = f'{SQS_URL}123456789012/celery-test'
 
         self.x.sqs_connection = Mock()
         self.x.sqs_connection._request_signer = Mock()
@@ -118,13 +183,28 @@ class test_AsyncSQSConnection(AWSCase):
             data=json.dumps({
                 **params,
                 "QueueUrl": queue_url
-            }),
+            }).encode(),
             headers={
                 'Content-Type': 'application/x-amz-json-1.0',
                 'X-Amz-Target': f'sqs.{operation_name}'
             },
         ))
 
+        prepared = req.prepare()  # without signing for test
+        assert prepared.method == 'POST'
+        assert prepared.url == SQS_URL
+        assert prepared.headers == {
+            'Content-Type': 'application/x-amz-json-1.0',
+            'X-Amz-Target': 'sqs.ReceiveMessage',
+            'Content-Length': mock.ANY,
+        }
+        assert json.loads(prepared.body) == {
+            'MaxNumberOfMessages': 10,
+            'AttributeNames': ['ApproximateReceiveCount'],
+            'WaitTimeSeconds': 20,
+            'QueueUrl': queue_url,
+        }
+
     def test_make_request__with_query_protocol(self):
         # Do the necessary mocking.
         self.x.sqs_connection = Mock()
@@ -137,14 +217,20 @@ class test_AsyncSQSConnection(AWSCase):
         operation = 'ReceiveMessage',
         params = {
             'MaxNumberOfMessages': 10,
-            'AttributeName.1': 'ApproximateReceiveCount',
             'WaitTimeSeconds': 20
         }
-        queue_url = f'{SQS_URL}/123456789012/celery-test'
+        pparams = {
+            'json': {'AttributeNames': ['ApproximateReceiveCount']},
+            'query': {'AttributeName.1': 'ApproximateReceiveCount'},
+        }
+        queue_url = f'{SQS_URL}123456789012/celery-test'
         verb = 'POST'
-        self.x.make_request(operation, params, queue_url, verb)
+
+        expect_params = {**params, 'AttributeName.1': 'ApproximateReceiveCount'}
+
+        self.x.make_request(operation, params, queue_url, verb, protocol_params=pparams)
         self.x._create_query_request.assert_called_with(
-            operation, params, queue_url, verb
+            operation, expect_params, queue_url, verb
         )
 
     def test_make_request__with_json_protocol(self):
@@ -159,14 +245,20 @@ class test_AsyncSQSConnection(AWSCase):
         operation = 'ReceiveMessage',
         params = {
             'MaxNumberOfMessages': 10,
-            'AttributeName.1': 'ApproximateReceiveCount',
             'WaitTimeSeconds': 20
         }
-        queue_url = f'{SQS_URL}/123456789012/celery-test'
+        pparams = {
+            'json': {'AttributeNames': ['ApproximateReceiveCount']},
+            'query': {'AttributeName.1': 'ApproximateReceiveCount'},
+        }
+
+        queue_url = f'{SQS_URL}123456789012/celery-test'
         verb = 'POST'
-        self.x.make_request(operation, params, queue_url, verb)
+        expect_params = {**params, 'AttributeNames': ['ApproximateReceiveCount']}
+
+        self.x.make_request(operation, params, queue_url, verb, protocol_params=pparams)
         self.x._create_json_request.assert_called_with(
-            operation, params, queue_url
+            operation, expect_params, queue_url
         )
 
     def test_create_queue(self):
@@ -211,11 +303,12 @@ class test_AsyncSQSConnection(AWSCase):
             queue, 'Expires', '3600', callback=self.callback,
         )
         self.x.get_status.assert_called_with(
-            'SetQueueAttribute', {
-                'Attribute.Name': 'Expires',
-                'Attribute.Value': '3600',
+            'SetQueueAttribute',
+            {}, queue.id, callback=self.callback,
+            protocol_params={
+                'json': {'Attributes': {'Expires': '3600'}},
+                'query': {'Attribute.Name': 'Expires', 'Attribute.Value': '3600'},
             },
-            queue.id, callback=self.callback,
         )
 
     def test_receive_message(self):
@@ -229,11 +322,14 @@ class test_AsyncSQSConnection(AWSCase):
         self.x.get_list.assert_called_with(
             'ReceiveMessage', {
                 'MaxNumberOfMessages': 4,
-                'AttributeName.1': 'ApproximateReceiveCount'
             },
             [('Message', AsyncMessage)],
             'http://aws.com', callback=self.callback,
             parent=queue,
+            protocol_params={
+                'json': {'AttributeNames': ['ApproximateReceiveCount']},
+                'query': {'AttributeName.1': 'ApproximateReceiveCount'},
+            },
         )
 
     def test_receive_message__with_visibility_timeout(self):
@@ -249,11 +345,14 @@ class test_AsyncSQSConnection(AWSCase):
             'ReceiveMessage', {
                 'MaxNumberOfMessages': 4,
                 'VisibilityTimeout': 3666,
-                'AttributeName.1': 'ApproximateReceiveCount',
             },
             [('Message', AsyncMessage)],
             'http://aws.com', callback=self.callback,
             parent=queue,
+            protocol_params={
+                'json': {'AttributeNames': ['ApproximateReceiveCount']},
+                'query': {'AttributeName.1': 'ApproximateReceiveCount'},
+            },
         )
 
     def test_receive_message__with_wait_time_seconds(self):
@@ -269,11 +368,14 @@ class test_AsyncSQSConnection(AWSCase):
             'ReceiveMessage', {
                 'MaxNumberOfMessages': 4,
                 'WaitTimeSeconds': 303,
-                'AttributeName.1': 'ApproximateReceiveCount',
             },
             [('Message', AsyncMessage)],
             'http://aws.com', callback=self.callback,
             parent=queue,
+            protocol_params={
+                'json': {'AttributeNames': ['ApproximateReceiveCount']},
+                'query': {'AttributeName.1': 'ApproximateReceiveCount'},
+            },
         )
 
     def test_receive_message__with_attributes(self):
@@ -287,13 +389,37 @@ class test_AsyncSQSConnection(AWSCase):
         )
         self.x.get_list.assert_called_with(
             'ReceiveMessage', {
-                'AttributeName.1': 'foo',
-                'AttributeName.2': 'bar',
                 'MaxNumberOfMessages': 4,
             },
             [('Message', AsyncMessage)],
             'http://aws.com', callback=self.callback,
             parent=queue,
+            protocol_params={
+                'json': {'AttributeNames': ['foo', 'bar']},
+                'query': {'AttributeName.1': 'foo', 'AttributeName.2': 'bar'},
+            },
+        )
+
+    def test_receive_message__with_fetch_attributes(self):
+        queue = Mock(name='queue')
+        self.x.fetch_message_attributes = ["DifferentAttribute1", "Another2"]
+        self.x.receive_message(
+            queue,
+            self.x.get_queue_url('queue'),
+            4,
+            callback=self.callback,
+        )
+        self.x.get_list.assert_called_with(
+            'ReceiveMessage', {
+                'MaxNumberOfMessages': 4,
+            },
+            [('Message', AsyncMessage)],
+            'http://aws.com', callback=self.callback,
+            parent=queue,
+            protocol_params={
+                'json': {'AttributeNames': ['DifferentAttribute1', 'Another2']},
+                'query': {'AttributeName.1': 'DifferentAttribute1', 'AttributeName.2': 'Another2'},
+            },
         )
 
     def MockMessage(self, id=None, receipt_handle=None, body=None):
@@ -328,13 +454,17 @@ class test_AsyncSQSConnection(AWSCase):
                     self.MockMessage('2', 'r2')]
         self.x.delete_message_batch(queue, messages, callback=self.callback)
         self.x.get_object.assert_called_with(
-            'DeleteMessageBatch', {
-                'DeleteMessageBatchRequestEntry.1.Id': '1',
-                'DeleteMessageBatchRequestEntry.1.ReceiptHandle': 'r1',
-                'DeleteMessageBatchRequestEntry.2.Id': '2',
-                'DeleteMessageBatchRequestEntry.2.ReceiptHandle': 'r2',
-            },
+            'DeleteMessageBatch', {},
             queue.id, verb='POST', callback=self.callback,
+            protocol_params={
+                'json': {'Entries': [{'Id': '1', 'ReceiptHandle': 'r1'}, {'Id': '2', 'ReceiptHandle': 'r2'}]},
+                'query': {
+                    'DeleteMessageBatchRequestEntry.1.Id': '1',
+                    'DeleteMessageBatchRequestEntry.1.ReceiptHandle': 'r1',
+                    'DeleteMessageBatchRequestEntry.2.Id': '2',
+                    'DeleteMessageBatchRequestEntry.2.ReceiptHandle': 'r2',
+                },
+            },
         )
 
     def test_send_message(self):
@@ -398,19 +528,25 @@ class test_AsyncSQSConnection(AWSCase):
             queue, messages, callback=self.callback,
         )
 
-        def preamble(n):
-            return '.'.join(['ChangeMessageVisibilityBatchRequestEntry', n])
-
-        self.x.get_object.assert_called_with(
-            'ChangeMessageVisibilityBatch', {
-                preamble('1.Id'): '1',
-                preamble('1.ReceiptHandle'): 'r1',
-                preamble('1.VisibilityTimeout'): 303,
-                preamble('2.Id'): '2',
-                preamble('2.ReceiptHandle'): 'r2',
-                preamble('2.VisibilityTimeout'): 909,
-            },
+        self.x.get_object.assert_called_once_with(
+            'ChangeMessageVisibilityBatch', {},
             queue.id, verb='POST', callback=self.callback,
+            protocol_params={
+                'json': {
+                    'Entries': [
+                        {'Id': '1', 'ReceiptHandle': 'r1', 'VisibilityTimeout': 303},
+                        {'Id': '2', 'ReceiptHandle': 'r2', 'VisibilityTimeout': 909},
+                    ],
+                },
+                'query': {
+                    'ChangeMessageVisibilityBatchRequestEntry.1.Id': '1',
+                    'ChangeMessageVisibilityBatchRequestEntry.1.ReceiptHandle': 'r1',
+                    'ChangeMessageVisibilityBatchRequestEntry.1.VisibilityTimeout': '303',
+                    'ChangeMessageVisibilityBatchRequestEntry.2.Id': '2',
+                    'ChangeMessageVisibilityBatchRequestEntry.2.ReceiptHandle': 'r2',
+                    'ChangeMessageVisibilityBatchRequestEntry.2.VisibilityTimeout': '909',
+                },
+            },
         )
 
     def test_get_all_queues(self):
@@ -480,3 +616,20 @@ class test_AsyncSQSConnection(AWSCase):
             'RemovePermission', {'Label': 'label'}, queue.id,
             callback=self.callback,
         )
+
+    def test_query_protocol_encoding(self):
+        assert _query_object_encode({}) == {}
+
+        assert _query_object_encode({'Simple': 'String'}) == {'Simple': 'String'}
+
+        assert _query_object_encode({'NumbersToString': 123}) == {'NumbersToString': '123'}
+
+        assert _query_object_encode({'AttributeName': ['A', 'B']}) == {
+            'AttributeName.1': 'A',
+            'AttributeName.2': 'B',
+        }
+
+        assert _query_object_encode({'Grandparent': [{'Parent': {'Child': '1', 'Sibling': 2}}]}) == {
+            'Grandparent.1.Parent.Child': '1',
+            'Grandparent.1.Parent.Sibling': '2',
+        }
