@@ -12,7 +12,7 @@ from t.mocks import Transport
 class ExchangeCase:
     type = None
 
-    def setup(self):
+    def setup_method(self):
         if self.type:
             self.e = self.type(Connection(transport=Transport).channel())
 
@@ -70,8 +70,8 @@ class test_Topic(ExchangeCase):
         ('stock.us.*', None, 'rBar'),
     ]
 
-    def setup(self):
-        super().setup()
+    def setup_method(self):
+        super().setup_method()
         self.table = [(rkey, self.e.key_to_pattern(rkey), queue)
                       for rkey, _, queue in self.table]
 
@@ -115,8 +115,8 @@ class test_TopicMultibind(ExchangeCase):
         ('#', None, 'rFoo'),
     ]
 
-    def setup(self):
-        super().setup()
+    def setup_method(self):
+        super().setup_method()
         self.table = [(rkey, self.e.key_to_pattern(rkey), queue)
                       for rkey, _, queue in self.table]
 
